﻿local COLOR_WHITE = "ffffff"
local COLOR_COPPER = "eda55f"
local COLOR_SILVER = "c7c7cf"
local COLOR_GOLD = "ffd700"
local CraftBot_Debug_Mode = false

CraftBot_On = true
CraftBot_PlayerIndex = 0
CraftBot_Data = {}

function CraftBot_OnLoad()
	CraftBot:RegisterEvent("ADDON_LOADED")
end

function CraftBot_OnEvent()
	if (event == "CHAT_MSG_WHISPER") then
		CraftBot_OnWhisper(arg1, arg2)
	elseif (event == "TRADE_SKILL_UPDATE") then
		CraftBot_Scan()
	elseif (event == "ADDON_LOADED" and arg1 == "CraftBot") then
		SlashCmdList["CRAFTBOT"] = CraftBot_SlashHandler
		SLASH_CRAFTBOT1 = "/craftbot"
		SLASH_CRAFTBOT2 = "/cb"
		local name = UnitName("player")
		local realm = GetRealmName()
		if #(CraftBot_Data) > 0 then
			for i=1, #(CraftBot_Data) do
				if CraftBot_Data[i].name == name and CraftBot_Data[i].realm == realm then
					CraftBot_PlayerIndex = i
					break
				end
			end
		end
		if CraftBot_PlayerIndex == 0 then
			local playerData = {
				["name"] = name,
				["realm"] = realm,
				["skills"] = {}
			}
			table.insert(CraftBot_Data, playerData)
			CraftBot_PlayerIndex = #(CraftBot_Data)
		end
		CountIt = 0
		CraftBot_Activate(CraftBot_On)
	end
end

function CraftBot_SlashHandler(msg)
	local words = {}
	for word in string.gmatch(msg, "%w+") do
		table.insert(words, word)
	end
	if #(words) then
		local cmd = words[1]
		if (cmd == "on") then
			CraftBot_Activate(true)
		elseif (cmd == "off") then
			CraftBot_Activate(false)
		end
	else
		CraftBot_Activate(not CraftBot_On)
	end
end

function CraftBot_Scan()

	local skillName, currentLevel, maxLevel = GetTradeSkillLine()
	if not skillName then
		return
	end
	local skillIndex = 0

	if #(CraftBot_Data[CraftBot_PlayerIndex].skills) > 0 then
		for i=1, #(CraftBot_Data[CraftBot_PlayerIndex].skills) do
			if CraftBot_Data[CraftBot_PlayerIndex].skills[i].name == skillName then
				skillIndex = i
				CraftBot_Data[CraftBot_PlayerIndex].skills[i].level = currentLevel
				break
			end
		end
	end
	
	if skillIndex == 0 then
		local skillNew = {
			["name"] = skillName,
			["specialization"] = "",
			["level"] = currentLevel,
			["data"] = {}
		}
		table.insert(CraftBot_Data[CraftBot_PlayerIndex].skills, skillNew)
		skillIndex = #(CraftBot_Data[CraftBot_PlayerIndex].skills)
	end

	local craftNum = GetNumTradeSkills() - #(CraftBot_Split(BuildListString(GetTradeSkillSubClasses()), ","))

	if craftNum == nil or craftNum <= #(CraftBot_Data[CraftBot_PlayerIndex].skills[skillIndex].data) then
		return
	end

	CraftBot_Data[CraftBot_PlayerIndex].skills[skillIndex].data = {}
	local numScanError = 0
	for i=1, craftNum do
		local thisScanError = false
		local craftData = {
	 		["craftName"] = "",
	 		["craftSubSpellName"] = "",
	 		["craftLink"] = "",
	 		["craftDesc"] = "",
	 		["qty"] = {
				["min"] = 0,
				["max"] = 0
			},
	 		["reagents"] = {},
		}
		local craftName, skillType, numAvailable, isExpanded = GetTradeSkillInfo(i)
		if GetTradeSkillItemLink(i) then
			craftData.craftName = craftName
			craftData.craftLink = GetTradeSkillItemLink(i)
			local itemstats = GetTradeSkillItemStats(i)
			if itemstats ~= nil then
				for x=1, #(itemstats) do
					if itemstats[x] then
						if string.find(itemstats[x], "|", 0, true) then
						else
							craftData.craftDesc = itemstats[x] .. " " .. craftData.craftDesc
						end
					end
				end
			end
			craftData.qty.min, craftData.qty.max = GetTradeSkillNumMade(i)
			for m=1, GetTradeSkillNumReagents(i) do
				local _, _, reagentCount, _ = GetTradeSkillReagentInfo(i, m)
				local reagentLink = GetTradeSkillReagentItemLink(i,m)
				if (reagentLink) then
					table.insert(craftData.reagents, { reagentLink, reagentCount })
				else
					thisScanError = true
					numScanError = numScanError + 1
					CraftBot_Print("CraftBot scan error: " .. craftData.craftLink)
					break
				end
			end
			if not thisScanError then
				table.insert(CraftBot_Data[CraftBot_PlayerIndex].skills[skillIndex].data, craftData)
			end
		end
	end
	if numScanError == 0 then
		CraftBot_Print(CRAFTBOT_SCAN_COMPLETE)
	else
		CraftBot_Print(format(CRAFTBOT_SCAN_COMPLETE_ERROR, numScanError))
	end
	CraftBot_Print(format(CRAFTBOT_SCAN_RECORDED,(craftNum - numScanError)))
end

function CraftBot_Activate(state)
	if (state) then
		CraftBot:RegisterEvent("CHAT_MSG_WHISPER")
		CraftBot:RegisterEvent("TRADE_SKILL_UPDATE")
		CraftBot_On = true
		CraftBot_Print(CRAFTBOT_ON)
	else
		CraftBot:UnregisterEvent("CHAT_MSG_WHISPER")
		CraftBot:UnregisterEvent("TRADE_SKILL_UPDATE")
		CraftBot_On = false
		CraftBot_Print(CRAFTBOT_OFF)
	end
	if IsAddOnLoaded("CraftBotFu") then
		CraftBotFu:Update()
	end
end

function CraftBot_SendChat(user, text)
	if user == UnitName("player") then
		CraftBot_Print(text)
	else
		SendChatMessage(text, "WHISPER", GetDefaultLanguage("player"), user)
	end
end

function CraftBot_OnWhisper(text, user)
	if text == "cbhelp" or text == "cbsearch" then
		local day, month = tonumber(date("%d")), tonumber(date("%m"))
		for i = 1, #(CraftBot_Holidays) do
			for x = 1, #(CraftBot_Holidays[i].dates) do
				if month == CraftBot_Holidays[i].dates[x][1] and day == CraftBot_Holidays[i].dates[x][2] then
					CraftBot_SendChat(user, CraftBot_Holidays[i].greetings)
					break
				end
			end
		end

		CraftBot_SendChat(user,  format(CRAFTBOT_WELCOME, UnitName("player")))

		CraftBot_SendChat(user, CRAFTBOT_HELP)
		return
	end

	local X = 0
	local tmp2 = strlower(text)
	local Y = nil

	--strip "cbsearch " from the string
	local p, n = string.find(tmp2, "cbsearch ")
	if p == nil and n == nil then
		local p, n = string.find(tmp2, "cbreagents ")
		if p == nil and n == nil then
			return
		else
			Y = string.sub(tmp2, n+1, strlen(tmp2))
		end
	else
		tmp2 = string.sub(tmp2, n+1, strlen(tmp2))
		Y = nil
	end

	if Y == nil then -- requesting search of description and recipe name
		for i=1, #(CraftBot_Data) do
			-- only if same realm
			if CraftBot_Data[i].realm == GetRealmName() then
				for x=1, #(CraftBot_Data[i].skills) do
					for y=1, #(CraftBot_Data[i].skills[x].data) do
						local craftData = CraftBot_Data[i].skills[x].data[y]
						local foo2 = nil
						local craftName, craftLink, craftDesc, craftMin, craftMax = craftData.craftName, craftData.craftLink, craftData.craftDesc, craftData.qty.min, craftData.qty.max

						local foo = string.find(strlower(craftName),tmp2)

						if craftDesc ~= nil or craftDesc ~= "" then
							foo2 = string.find(strlower(craftDesc),tmp2)
						end

						if foo ~= nil or foo2 ~= nil then
							X = 1
							CountIt = CountIt + 1
							if craftMin ~= craftMax then
								CraftBot_SendChat(user, format("[%d.%d.%d] ", i, x, y) .. craftLink .. " Qty: " .. craftMin .. "min/" ..craftMax .. "max")
							else
								CraftBot_SendChat(user, format("[%d.%d.%d] ", i, x, y) .. craftLink)
							end
						end
					end
				end
			end
		end
		if X == 1 then
			CraftBot_SendChat(user,CRAFTBOT_HELP_MAT)
			CraftBot_Print(format(CRAFTBOT_RESULTS_OK,text,user,user,CountIt))
		else
			CraftBot_SendChat(user, CRAFTBOT_HELP_NOK)
			CraftBot_Print(format(CRAFTBOT_RESULTS_NOK,text,user,user))
		end
		CountIt = 0

	else --requesting ingredients
		CraftBot_SendReagents(Y, user)
	end
end

function CraftBot_SendReagents(msg, user)

	if not #(CraftBot_Data) then
		return
	end

	local playerIndex,skillIndex,craftIndex= 0,0,0
	local isDotFormat = false

	-- playerIndex.skillIndex.craftIndex format
	if string.find(msg,".") then
		local tmp = CraftBot_Split(msg,".")
		if #(tmp) ~= 3 then
		else
			playerIndex,skillIndex,craftIndex = tonumber(tmp[1]),tonumber(tmp[2]),tonumber(tmp[3])
			if playerIndex ~= nil and skillIndex ~= nil and craftIndex ~= nil then
				isDotFormat = true
			end
		end
	end

	if not isDotFormat then
		if string.find(msg, "|") then
			for i=1, #(CraftBot_Data) do
				-- only if same realm
				if CraftBot_Data[i].realm == GetRealmName() then
					for x=1, #(CraftBot_Data[i].skills) do
						for y=1, #(CraftBot_Data[i].skills[x].data) do
							if CraftBot_Data[i].skills[x].data[y].craftLink == msg then
								playerIndex,skillIndex,craftIndex = i,x,y
								isDotFormat = true
								break
							end
						end
					end
				end
			end
		else
			for i=1, #(CraftBot_Data) do
				-- only if same realm
				if CraftBot_Data[i].realm == GetRealmName() then
					for x=1, #(CraftBot_Data[i].skills) do
						for y=1, #(CraftBot_Data[i].skills[x].data) do
							if string.find(strlower(CraftBot_Data[i].skills[x].data[y].craftName), strlower(msg)) then
								playerIndex,skillIndex,craftIndex = i,x,y
								isDotFormat = true
								break
							end
						end
					end
				end
			end
		end
	end
	
	if isDotFormat then
		local reagentsData = CraftBot_Data[playerIndex].skills[skillIndex].data[craftIndex].reagents
		local reagents = " "
		local rtmp = nil
		local foo = false
		CraftBot_SendChat(user, "Reagents for " .. CraftBot_Data[playerIndex].skills[skillIndex].data[craftIndex].craftLink .. " :")
		for m=1, #(reagentsData) do
			local reagentLink, reagentCount = reagentsData[m][1], reagentsData[m][2]
			rtmp = reagents .. " " .. reagentCount .. " x " .. reagentLink
			if(strlen(rtmp) > 255) then
				CraftBot_SendChat(user, reagents)
				rtmp = "(continued) " .. reagentCount .. " x " .. reagentLink
				reagents = rtmp
				foo = true
			end
			reagents = rtmp
		end
		CraftBot_SendChat(user, reagents)
		CraftBot_Print(format(CRAFTBOT_MAT_OK,msg,user,user))
	else
		CraftBot_Print(format(CRAFTBOT_MAT_NOK,msg,user,user))
	end
end

function CraftBot_GetTooltipText()
	local text
	for i=1, #(CraftBot_Data) do
		-- only if same realm
		if CraftBot_Data[i].realm == GetRealmName() then
			local craftNum = #(CraftBot_Data[i].skills)
			if craftNum > 0 then
				if text ~= "" then
					text = text .. "\n"
				end
				text = text .. CraftBot_Data[i].name .. "\n"
				for x=1, craftNum do
					text = text .. CraftBot_Data[i].skills[x].name .. " (".. CraftBot_Data[i].skills[x].level .. "): " .. #(CraftBot_Data[i].skills[x].data)
					if x ~= craftNum then
						text = text .. "\n"
					end
				end
			end
		end
	end
	return text
end

function CraftBot_GetButtonText()
	if (CraftBot_On == false) then
		return CRAFTBOT_SMODE_OFF
	else
		return CRAFTBOT_SMODE_ON
	end
end

-- Support Functions
local function CraftBot_FormatMoney(amt)
	local gold = 0
	local silver = 0
	local copper = 0
	local formattedMoney = ""

	if amt < 100 then
		copper = amt
		if copper ~= 0 then
			formattedMoney = format("|cff%s%02d|r|cff%s%s|r", COLOR_WHITE, copper, COLOR_COPPER,"c")
		end
	else
		copper = mod(amt, 100)
		if copper ~= 0 then
			formattedMoney = format("|cff%s%02d|r|cff%s%s|r", COLOR_WHITE, copper, COLOR_COPPER,"c")
		end
		amt = amt - copper
		if amt < 10000 then
			silver = amt/100
			if silver ~= 0 then
				formattedMoney = format("|cff%s%02d|r|cff%s%s|r", COLOR_WHITE, silver, COLOR_SILVER,"s ") .. formattedMoney
			end
		else
			silver = mod(amt, 10000) / 100
			if silver ~= 0 then
				formattedMoney = format("|cff%s%02d|r|cff%s%s|r", COLOR_WHITE, silver, COLOR_SILVER,"s ") .. formattedMoney
				gold = (amt - (silver * 100))/ 10000
				formattedMoney = format("|cff%s%d|r|cff%s%s|r", COLOR_WHITE, gold, COLOR_GOLD,"g ") .. formattedMoney
			end
		end
	end
	return formattedMoney
end

function CraftBot_Split(text, separator) 
	local value
   	local x, y = 1
	local t = {}
	local regexpstring = "([^"..separator.."]+)"
	if (text and separator) then
		x, y, value = string.find(text, regexpstring, x)
	   	while (value) do
			table.insert(t, value)
			x = y + 1
			x, y, value = string.find(text, regexpstring, x)
	   	end
	end
	return t
end

function CraftBot_Print(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg)
end

function CraftBot_Debug(msg)
	if CraftBot_Debug_Mode then
		DEFAULT_CHAT_FRAME:AddMessage(msg)
	end
end